/*
 * CategoryView.cpp
 *
 *  Created on: 21.12.2009
 *      Author: apoi
 */

#include <QPushButton>
#include <QInputDialog>
#include <QHeaderView>
#include <QAction>
#include <QEvent>
#include <QKeyEvent>
#include <QApplication>

#include "BlogView.h"
#include "CategoryView.h"
#include "CheckableListItem.h"
#include "WPData.h"
#include "wpscrollarea.h"

#include "ScopedTimer.h"

static const char* KEmptyParent = "0";

CategoryView::CategoryView(BlogView *aBlogview, QWidget * aParent) : MDragWidget(aParent), iBlogview(aBlogview), iScrollArea(NULL)
{
    setWindowFlags(Qt::Window);
#ifdef Q_WS_MAEMO_5
    setAttribute(Qt::WA_Maemo5StackedWindow);
#endif
    setWindowTitle(tr("Categories"));
    iLayout = new QVBoxLayout(this);
    iLayout->setSpacing(0);
    iLayout->setContentsMargins(0, 0, 0, 0);
    setLayout(iLayout);

//#ifdef Q_OS_SYMBIAN
    // Prepare softkeys
    QAction* leftSoftKey = new QAction(tr("Select"), this);
    connect(leftSoftKey, SIGNAL(triggered()), this, SLOT(SaveSelection()));
    leftSoftKey->setSoftKeyRole(QAction::PositiveSoftKey);
    //addAction(leftSoftKey);
    QAction* rightSoftKey = new QAction(tr("Cancel"), this);
    connect(rightSoftKey, SIGNAL(triggered()), this, SLOT(CloseView()));
    rightSoftKey->setSoftKeyRole(QAction::NegativeSoftKey);
    //addAction(rightSoftKey);
//#endif

    // Finally add the data for WPCustomViewBasicData
    //
    iTitleText = tr("Select Category");
    iTabText = tr("Edit");
    iHappyAction = leftSoftKey;
    iSadAction = rightSoftKey;
}

CategoryView::~CategoryView()
{
    delete iLayout;
    delete iScrollArea;
}

void CategoryView::UpdateCategoryData(const WPDataBlog& aBlog)
{
    // XXX: This is god damn slow, do something about it
    // Worked around with QApplication::processEvents();
    // Remember that this screws up timers...

    TIME_SECTIONS_START
    TIME_NEW_SECTION("First half of function", 10)

    iWidgetMap.clear();
    iCategoryMap.clear();

    // Clear layout before repopulating
    ClearLayout(iLayout);

    delete iScrollArea;
    iScrollArea = new WPScrollArea();
    iScrollArea->setFocusPolicy(Qt::NoFocus);

    // Add button for new categories
    // XXX: leaked?
    QPushButton* addCategory = new QPushButton(tr("Add new category..."), this);
    connect(addCategory, SIGNAL(clicked()), this, SLOT(AddCategory()));

    // First, create a map of categories
    const WPDataCategory* category;
    QMap<QString, QStringList*> childMap;
    QListIterator<WPDataCategory> i(aBlog.iCategories);
    while (i.hasNext())
    {
        category = &(i.next());
        iCategoryMap.insert(category->iCategoryId, category);
        QStringList *childList = new QStringList;
        childMap.insert(category->iCategoryId, childList);
    }

    // Then, create a map of parent connections
    i.toFront();
    while (i.hasNext())
    {
        category = &(i.next());
        if (!category->iParentId.isEmpty() && category->iParentId != QLatin1String(KEmptyParent))
        {
            childMap.value(category->iParentId)->append(category->iCategoryId);
        }
    }

    TIME_NEW_SECTION("UpdateCategoryData Call a recursive function for all top-level widgets", 10)

    // Call a recursive function for all top-level widgets
    i.toFront();
    while (i.hasNext())
    {
        category = &(i.next());
        if (category->iParentId.isEmpty() || category->iParentId == QLatin1String(KEmptyParent))
        {
            CreateListItem(category->iCategoryId, 0, childMap);
        }
    }

    TIME_NEW_SECTION("UpdateCategoryData deletes", 10)

    // Remember to delete heap variables
    QMap<QString, QStringList*>::iterator k;
    for (k = childMap.begin(); k != childMap.end(); ++k)
        delete k.value();


    TIME_NEW_SECTION("UpdateCategoryData Do the old selection again...", 10)

    // Do the old selection again, and check if a new
    // category has just been added. If so, select it too.
    if (!iNewCategory.isEmpty())
    {
        SelectCategories(&iCurrentSelection);
        const WPDataCategory* category = GetCategoryByName(iNewCategory);
        if (category != 0)
        {
            SelectCategory(category->iCategoryId);
        }
        iNewCategory.clear();
        iCurrentSelection.clear();
    }

    TIME_NEW_SECTION("UpdateCategoryData Add widget 1", 10)
    iLayout->addWidget(iScrollArea);
    TIME_END_SECTION
    QApplication::processEvents();

    TIME_NEW_SECTION("UpdateCategoryData Add widget 2", 10)
    iLayout->addWidget(addCategory);
    TIME_END_SECTION
    QApplication::processEvents();

    TIME_NEW_SECTION("UpdateCategoryData Add stretch", 10)
    iScrollArea->AddStretch();
    TIME_END_SECTION
    QApplication::processEvents();

    TIME_NEW_SECTION("UpdateCategoryData Set active view", 10)

    // Focus update may be needed if this is the active view
    iBlogview->SetActiveView(BlogView::ECategoryView);
}

void CategoryView::CreateListItem(const QString& aItemId, int aCategoryLevel, QMap<QString, QStringList*>& aChildMap)
{
    QApplication::processEvents();
    CheckableListItem *item = new CheckableListItem(CheckableListItem::ECategory, aItemId, false, *this);
    item->InitCategoryItem(iCategoryMap.value(aItemId)->iCategoryName, aCategoryLevel);
    item->setMinimumHeight(50);
    iWidgetMap.insert(aItemId, item);
    iScrollArea->addWidget(item);
    QStringList *childList = aChildMap.value(aItemId);
    foreach (QString childId, *childList)
    {
        CreateListItem(childId, aCategoryLevel + 1, aChildMap);
    }
}

void CategoryView::SelectCategories(const QStringList* aCategories)
{
    if (aCategories)
    {
        iOriginalSelection = *aCategories;
    }
    else
    {
        iOriginalSelection.clear();
    }
        
    ClearSelection();
    foreach (QString categoryName, *aCategories)
    {
        const WPDataCategory* category = GetCategoryByName(categoryName);
        if (category != 0)
        {
            SelectCategory(category->iCategoryId);
        }
    }
}

const WPDataCategory* CategoryView::GetCategoryByName(const QString& aCategoryName)
{
    // We have no value->key mapping, so let's loop the list of categories
    const WPDataCategory* category = 0;
    QMap<QString, const WPDataCategory*>::const_iterator i = iCategoryMap.constBegin();
    while (i != iCategoryMap.constEnd()) {
        if (i.value()->iCategoryName.compare(aCategoryName) == 0)
        {
            category = i.value();
            break;
        }
        ++i;
    }
    return category;
}

void CategoryView::SelectCategory(const QString& aCategoryId)
{
    if (!aCategoryId.isEmpty())
    {
        iWidgetMap.value(aCategoryId)->SetCheckState(true);
    }
}

void CategoryView::ClearSelection()
{
    QStringList keyList = iWidgetMap.keys();
    foreach (QString key, keyList)
    {
        iWidgetMap.value(key)->SetCheckState(false);
    }
}

void CategoryView::SaveSelection()
{
    QStringList list;
    QStringList keyList = iWidgetMap.keys();
    foreach (QString key, keyList)
    {
        if (iWidgetMap.value(key)->IsChecked())
        {
            list << iCategoryMap.value(key)->iCategoryName;
        }
    }
    iBlogview->SelectCategories(&list);
//#ifdef Q_WS_MAEMO_5
//    close();
//#endif
}

void CategoryView::CloseView()
{
    iBlogview->SelectCategories(&iOriginalSelection);
}

void CategoryView::AddCategory()
{
    bool ok;
    iNewCategory = QInputDialog::getText(this,
                                         tr("Category name"),
                                         tr("Category name:"),
                                         QLineEdit::Normal, QString(), &ok);

    if (ok && !iNewCategory.isEmpty())
    {
        // Save the current selection, so that the refreshed view can show it correctly
        iCurrentSelection.clear();
        QStringList keyList = iWidgetMap.keys();
        foreach (QString key, keyList)
        {
            if (iWidgetMap.value(key)->IsChecked())
            {
                iCurrentSelection << iCategoryMap.value(key)->iCategoryName;
            }
        }

        emit NewCategory(iNewCategory);
    }
}

bool CategoryView::MDragWidgetIsDragging() const
{
    return iScrollArea->IsDragging();
}

QSize CategoryView::VisibleSize() const
{
    return iScrollArea->GetVisibleContentSize();
}

void CategoryView::setFocus()
{
    QLayoutItem *item = iScrollArea->itemAt(0);
    if (item)
    {
        QWidget *widgetItem = item->widget();
        if (widgetItem)
        {
            widgetItem->setFocus();
        }
        else
        {
            iScrollArea->setFocus();
        }
    }
    else
    {
        iScrollArea->setFocus();
    }
}

/*
 * Recursively clear layouts. This is needed, since
 * otherwise the old elements will linger below new ones.
 */
void CategoryView::ClearLayout(QLayout* aLayout)
{
    QLayoutItem *item;
    while ((item = aLayout->itemAt(0)) != 0)
    {
        // If the item is a widget, delete it
        QWidget *widget = item->widget();
        if (widget)
        {
            widget->deleteLater();
        }
        else
        {
            // If the item is a layout, recurse to clear it
            QLayout *layout = item->layout();
            if (layout)
            {
                ClearLayout(layout);
            }
        }
        aLayout->removeItem(item);
    }
}
