/*
 * DialogController.h
 *
 *  Created on: 5.1.2010
 *      Author: apoi
 */

#ifndef DIALOGCONTROLLER_H_
#define DIALOGCONTROLLER_H_

#include <QObject>
#include <QWidget>

#include "WPNetworkEngine.h"

class BlogView;
class ErrorDialog;
class ProgressDialog;
   
class DialogController : public QObject
{
    Q_OBJECT

public:
    DialogController(BlogView *aParent, bool cancellable = false);
    ~DialogController();

    /**
     * Display a modal progress dialog for the currently executing set of tasks, hiding it once
     * all tasks complete.
     */
    void StartTaskProgress ();


public slots:
    void ShowInfoNote (const QString &title, const QString &message);
    void ShowErrorNote (const QString &error);

    /* Network -> */
    void TaskStarted (TWPNetworkEngineCommand command, bool foreground = false);
    void TaskFinished (TWPNetworkEngineCommand command);
    void TaskFailed (TWPNetworkEngineCommand command, QString msg);

    void ExitProgress();
    void ErrorAccepted();
   
signals:
    void TaskCancelled();

private:
    /**
     * Display modal progress dialog for currently executing task
     */
    void ShowTaskProgress ();

    void SetProgressMessage();
    
    void CheckStatus();

private slots:
    void DelayedProgressDialogClose();

private:
    BlogView *iParent;
    ErrorDialog *iError;
    ProgressDialog *iProgress;
    
    /** Show task progress until idle? */
    bool iShowTaskProgress;

    /** List of pending commands*/
    QList<TWPNetworkEngineCommand> iCommandList;

    /** Map from command id -> description XXX: static */
    QStringList iCommandMessages;
    
    /** Is the button action cancel */
    bool cancellable;

};

#endif /* DIALOGCONTROLLER_H_ */
