/*
 * PagesView.cpp
 *
 *  Created on: 4.11.2009
 *      Author: thyv
 */

#include <QGraphicsLinearLayout>
#include <QDateTime>
#include <QMenu>
#include <QDesktopWidget>

#include "PagesView.h"
#include "WPData.h"
#include "BlogView.h"
#include "wppagegraphicslist.h"
#include "WPNetworkEngine.h"
#include "textlistitem.h"

PagesView::PagesView(BlogView* aParent, const QGraphicsView& aGraphicsView, PannableView* aPannableView)
:   iParent(aParent), iGraphicsView(aGraphicsView), iPannableView(aPannableView)
{
    iListbox = NULL;
    iLayout = new QGraphicsLinearLayout(Qt::Vertical);
    iLayout->setSpacing(0);
    iLayout->setContentsMargins(0, 0, 0, 0);
    setLayout(iLayout);
}

PagesView::~PagesView()
{
    delete iListbox;
}

void PagesView::resizeEvent(QGraphicsSceneResizeEvent* /*event*/)
{
    // Set maximum width for the contained items, this
    // will reduce the amount of redraws
    if (iListbox)
    {
        iListbox->setGeometry(0, 0, size().width(), size().height());
    }
}

void PagesView::UpdatePageData(const WPDataBlog& aBlog)
{
    // clean
    while (iLayout->itemAt(0))
        iLayout->removeAt(0);

    delete iListbox;

    iListbox = new WPPageGraphicsList(aBlog.GetPages(), iGraphicsView, iParent, iPannableView);
    connect(iListbox, SIGNAL(OpenClicked(WPPage)), this, SIGNAL(OpenClicked(WPPage)));
    connect(iListbox, SIGNAL(EditClicked(WPPage)), this, SIGNAL(EditClicked(WPPage)));
    // Delete handled internally 
    iLayout->addItem(iListbox);

    // signals
    connect(
       &aBlog,  SIGNAL(PageAdded(WPPage)), 
        iListbox, SLOT(PageAdded(WPPage))
    );
    connect(
       &aBlog,  SIGNAL(PageChanged(WPPage)), 
        iListbox, SLOT(PageChanged(WPPage))
    );
    connect(
       &aBlog,  SIGNAL(PageRemoved(WPPage)), 
        iListbox, SLOT(PageRemoved(WPPage))
    );

}

void PagesView::setFocus()
{
    if (iListbox)
        iListbox->setFocus();
}


