/*
 * PostsView.cpp
 *
 *  Created on: 4.11.2009
 *      Author: thyv
 */

#include <QGraphicsLinearLayout>
#include <QDateTime>
#include <QMenu>
#include <QDesktopWidget>

#include "PostsView.h"
#include "WPData.h"
#include "BlogView.h"
#include "wppostgraphicslist.h"
#include "WPNetworkEngine.h"
#include "textlistitem.h"

PostsView::PostsView(BlogView* aParent, const QGraphicsView& aGraphicsView, PannableView* aPannableView)
:   iParent(aParent), iGraphicsView(aGraphicsView), iPannableView(aPannableView)
{
    iListbox = NULL;
    iLayout = new QGraphicsLinearLayout(Qt::Vertical);
    iLayout->setSpacing(0);
    iLayout->setContentsMargins(0, 0, 0, 0);
    setLayout(iLayout);
}

PostsView::~PostsView()
{
    delete iListbox;
}

void PostsView::resizeEvent(QGraphicsSceneResizeEvent* /*e*/)
{
    // Set maximum width for the contained items, this
    // will reduce the amount of redraws
    if (iListbox)
    {
        iListbox->setGeometry(0, 0, size().width(), size().height());
    }
}

void PostsView::UpdatePostData(const WPDataBlog& aBlog)
{
    // clean
    while (iLayout->itemAt(0))
        iLayout->removeAt(0);
    
    delete iListbox;

    iListbox = new WPPostGraphicsList(aBlog.GetPosts(), iGraphicsView, iParent, iPannableView);
    connect(iListbox, SIGNAL(OpenClicked(WPPost)), this, SIGNAL(OpenClicked(WPPost)));
    connect(iListbox, SIGNAL(EditClicked(WPPost)), this, SIGNAL(EditClicked(WPPost)));
    connect(iListbox, SIGNAL(DeleteClicked(WPPost)), this, SIGNAL(DeleteClicked(WPPost)));
    iLayout->addItem(iListbox);
    iParent->SetActiveView();

    // signals
    connect(
       &aBlog,  SIGNAL(PostAdded(WPPost)), 
        iListbox, SLOT(PostAdded(WPPost))
    );
    connect(
       &aBlog,  SIGNAL(PostChanged(WPPost)), 
        iListbox, SLOT(PostChanged(WPPost))
    );
    connect(
       &aBlog,  SIGNAL(PostRemoved(WPPost)), 
        iListbox, SLOT(PostRemoved(WPPost))
    );

}

void PostsView::setFocus()
{
    if (iListbox)
        iListbox->setFocus();
}


