
#include <QGraphicsScene>
#include <QGraphicsProxyWidget>
#include <QScrollBar>
#include <QApplication>

#include "BlogView.h"
#include "commentsgraphicsview.h"
#include "commentsview.h"
#include "WPNetworkEngine.h"
#include "enume/pannableview.h"

static const int KDecelerationTime = 500; // Time in milliseconds it takes to brake

CommentsGraphicsView::CommentsGraphicsView(BlogView& aBlogView, QWidget* aParent)
: QGraphicsView(aParent), iBlogView(aBlogView)
{
    iScene = new QGraphicsScene(this);
    setScene(iScene);
    iCommentsView = new CommentsView(&iBlogView, *this, iPannableView);
    iPannableView = new PannableView(Qt::Vertical);
    iPannableView->setPos(0,0);
    iPannableView->setWidget(iCommentsView);
    iPannableView->enableScrollIndicator(true);
    iScene->addItem(iPannableView);
    iCommentsView->setContentsMargins(0.0, 0.0, 0.0, 0.0);
    QRect rect(geometry());
    //iScene->setSceneRect(rect);
    setDragMode(QGraphicsView::NoDrag);
    
    setGeometry(rect);
    
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    //QLinearGradient ql(0, 0, 300, 300);
    //ql.setColorAt(0.0, QColor(61, 165, 199));
    //ql.setColorAt(1.0, QColor(36, 80, 94));
    setBackgroundBrush(QColor(41, 144, 178));
    setCacheMode(QGraphicsView::CacheBackground);
}

CommentsGraphicsView::~CommentsGraphicsView()
{
}

void CommentsGraphicsView::resizeEvent(QResizeEvent *e)
{
    QGraphicsView::resizeEvent(e);
    QRectF rect(geometry());
    QSize size(viewport()->size());
    rect.setSize(size);

    iCommentsView->setMinimumWidth(0);
    iCommentsView->setMaximumWidth(size.width());
    iCommentsView->resize(size);
    iPannableView->setMinimumSize(size);
    iPannableView->setPreferredSize(size);
    iPannableView->setMaximumSize(size);
}

void CommentsGraphicsView::mousePressEvent(QMouseEvent* aEvent)
{
    aEvent->accept();
    QGraphicsView::mousePressEvent(aEvent);
}

