
#include <QList>
#include <QGraphicsLinearLayout>
#include <QGraphicsSceneResizeEvent>

#include "commentsview.h"
#include "BlogView.h"
#include "WPData.h"
#include "wpcommentgraphicslist.h"
#include "enume/pannableview.h"

CommentsView::CommentsView(BlogView* aParent, const QGraphicsView& aGraphicsView, PannableView* aPannableView) 
: iParent(aParent), iGraphicsView(aGraphicsView), iPannableView(aPannableView)
{
    iListbox = NULL;
    iLayout = new QGraphicsLinearLayout(Qt::Vertical);
    iLayout->setSpacing(0);
    iLayout->setContentsMargins(0, 0, 0, 0);
    setLayout(iLayout);
}

CommentsView::~CommentsView()
{
    delete iListbox;
}

void CommentsView::resizeEvent(QGraphicsSceneResizeEvent* /*e*/)
{
    // Set maximum width for the contained items, this
    // will reduce the amount of redraws
    if (iListbox)
    {
        iListbox->setGeometry(0, 0, size().width(), size().height());
    }
}

void CommentsView::UpdateCommentsData(const WPDataBlog& aBlog)
{
    qDebug("WP::CommentsView::UpdateCommentsData");

    // clean
    while (iLayout->itemAt(0))
        iLayout->removeAt(0);
    
    delete iListbox;

    iListbox = new WPCommentGraphicsList(
        aBlog.GetComments(),
        iParent,
        iGraphicsView,
        iPannableView);
    
    connect(iListbox, SIGNAL(ReplyToComment(WPComment)), iParent, SLOT(ReplyToComment(WPComment)));

    iLayout->addItem(iListbox);

    // signals
    connect(
       &aBlog,  SIGNAL(CommentAdded(WPComment)), 
        iListbox, SLOT(CommentAdded(WPComment))
    );
    connect(
       &aBlog,  SIGNAL(CommentChanged(WPComment)), 
        iListbox, SLOT(CommentChanged(WPComment))
    );
    connect(
       &aBlog,  SIGNAL(CommentRemoved(WPComment)), 
        iListbox, SLOT(CommentRemoved(WPComment))
    );

    iParent->SetActiveView();
}

void CommentsView::setFocus()
{
    if (iListbox)
        iListbox->setFocus();
}
