#include "wpbloggraphicslist.h"
#include "wpbloggraphicslistitem.h"
#include "PopupMenu.h"
#include "BlogView.h"
#include "wpconfirmdialog.h"

#include <QAction>
#include <QPushButton>
#include <QGraphicsProxyWidget>
#include <QGraphicsLinearLayout>

WPBlogGraphicsList::WPBlogGraphicsList(BlogView* aBlogView, const QGraphicsView& aGraphicsView,
	PannableView* aPannableView)
  : WPGraphicsList(aBlogView, aGraphicsView, aPannableView)
  , iBlogView(aBlogView)
{
	 // TODO find a proper way to align menu icon and text side by side.
	iActionOpen = new QAction(iThemeData.iContextMenuIcons[ThemeData::EOpen],tr("    Open"), this);
	iActionEdit = new QAction(iThemeData.iContextMenuIcons[ThemeData::EEdit],tr("    Edit"), this);
	iActionDelete = new QAction(iThemeData.iContextMenuIcons[ThemeData::ERemove],tr("    Delete"), this);
	
	connect(iActionOpen,   SIGNAL(triggered()), this, SLOT(Open()));
	connect(iActionEdit,   SIGNAL(triggered()), this, SLOT(Edit()));
	connect(iActionDelete, SIGNAL(triggered()), this, SLOT(Delete()));
	
	iNewBlogButton = new QPushButton;
	iNewBlogButton->setText(tr("Add new blog"));
	iNewBlogProxy = new QGraphicsProxyWidget(this);
	iNewBlogProxy->setWidget(iNewBlogButton);
	iNewBlogButton->setObjectName(QString::fromLatin1("newBlogLabel"));
	//iNewBlogButton->setFixedHeight(KTitleDefaultHeight);
	//iNewBlogButton->setStyleSheet(QLatin1String(KButtonStyleSheet));
	connect(iNewBlogButton, SIGNAL(clicked(bool)), this, SLOT(AddNewBlog()));
    
    iLayout->setContentsMargins(5, 5, 5, 0);
	iLayout->addItem(iNewBlogProxy);
	iItems.append(NULL);    // placeholder
	
	iLayout->addStretch();
}

WPBlogGraphicsList::~WPBlogGraphicsList()
{

}

void WPBlogGraphicsList::UpdateData(const WPDataUsersBlogs& aUsersBlogs)
{
	int const firstIndex = FirstFocusableItemIndex();
	while(iItems.count() > firstIndex)
		{
		iLayout->removeItem(iItems.last());
		iItems.removeLast();
		}
		
	for (QListIterator<WPDataBlog *> i(aUsersBlogs.GetBlogs()); i.hasNext(); ) {
		AddBlog(*i.next());
	}
	
	iTitle = tr("Blogs (%1)") .arg(aUsersBlogs.GetBlogs().size());
}

void WPBlogGraphicsList::AddBlog(const WPDataBlog& aBlog)
{
	WPBlogGraphicsListItem* item = new WPBlogGraphicsListItem(aBlog, iBusyIndicator, iThemeData);
	connect(item, SIGNAL(Selected(WPGraphicsListItem*, QPoint)), this, SLOT(ShowContextMenu(WPGraphicsListItem*, QPoint)));

	iItems.append(item);
	iLayout->insertItem(iItems.size() - 1, item);
}

void WPBlogGraphicsList::PopulatePopupMenuForCurrentItem(PopupMenu* aPopupMenu)
{
	aPopupMenu->addAction(iActionOpen);
	aPopupMenu->addAction(iActionEdit);
	aPopupMenu->addAction(iActionDelete);
}

void WPBlogGraphicsList::AddNewBlog()
{
    iBlogView->ShowNewBlogView();
}

void WPBlogGraphicsList::Open()
{
    iBlogView->OpenBlog(CurrentBlogId());
#ifdef Q_WS_MAEMO_5
    close();
#endif
}

void WPBlogGraphicsList::Edit()
{
    iBlogView->ShowEditBlogView(CurrentBlogId());
}

void WPBlogGraphicsList::Delete()
{
    WP_CONFRIM_AND_RETURN_ON_FALSE;

    if (iBlogView->DeleteBlog(CurrentBlogId()))
    {
        iItems.removeOne(iCurrentItem);
    }
}

QString const WPBlogGraphicsList::CurrentBlogId()
	{
	return CurrentItem()->getBlog().LocalId();
	}
