#ifndef WPCOMMENTGRAPHICSLIST_H
#define WPCOMMENTGRAPHICSLIST_H

#include "wpgraphicslist.h"
#include "WPData.h"

class QLabel;
class QVBoxLayout;

class WPCommentGraphicsListItem;
class QGraphicsProxyWidget;
class BlogView;

class WPCommentGraphicsList : public WPGraphicsList
{
    Q_OBJECT

public:
    WPCommentGraphicsList(QListIterator<WPComment> aData, BlogView* aBlogView, const QGraphicsView& aGraphicsView,
        PannableView* aPannableView, bool aShowFullComment = false);
    ~WPCommentGraphicsList();

signals:
    void ReplyToComment(WPComment aComment);

public slots:
    void GravatarUpdated (const QString &email, const QPixmap &pixmap);
    
    void CommentChanged(WPComment aComment);
    void CommentRemoved(WPComment aComment);
    void CommentAdded(WPComment aComment);
    
private slots:
    void Approve();
    void Unapprove();
    void View();
    void Reply();
    void Delete();
    void Spam();

private:
    void AddComment(WPComment aComment, bool aShowFullComment);
    void RemoveComment (WPComment aComment);
    
    void SetLabelText();
    
    void DecrementUnapproved();
    void DecrementApproved();
    
    void IncrementUnapproved();
    void IncrementApproved();
    
private: // WPGraphicsList implementation
    void PopulatePopupMenuForCurrentItem(PopupMenu* aPopupMenu);
    int FirstFocusableItemIndex() { return 1; }
    
private:
    // Ugly workarounds because QObject does not support templates...
    WPCommentGraphicsListItem * ItemAt(int i) 
    	{ return reinterpret_cast<WPCommentGraphicsListItem *> (iItems[i]); }
    WPCommentGraphicsListItem * CurrentItem()
        { return reinterpret_cast<WPCommentGraphicsListItem *> (iCurrentItem); }

private: // Data
    WPDataGravatars*        iGravatars;
    
    QLabel* iUnapprovedLabel;
    QGraphicsProxyWidget* iUnapprovedProxy;
    QLabel* iApprovedLabel;
    QGraphicsProxyWidget* iApprovedProxy;
    int iUnapproved;
    int iApproved;

    QAction* iActionApprove;
    QAction* iActionUnapprove;
    QAction* iActionView;
    QAction* iActionHide;
    QAction* iActionReply;
    QAction* iActionDelete;
    QAction* iActionSpam;
};

#endif // WPCOMMENTGRAPHICSLIST_H
