
#include <QGraphicsGridLayout>
#include <QGraphicsLinearLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsSceneMouseEvent>

#include <QApplication>
#include <QEvent>
#include <QPoint>
#include <QKeyEvent>
#include <QRect>
#include <QDesktopWidget>
#include <QPainter>

#include "clippinglabel.h"
#include "WPData.h"
#include "wpcommentgraphicslist.h"
#include "wpcommentgraphicslistitem.h"
#include "WPNetworkEngine.h"
#include "MDragWidget.h"
#include "themedatastorage.h"
#include "busyindicator.h"
#include "wpcommongeometry.h"

static const int KCommentDefaultHeight = 57;

WPCommentGraphicsListItem::WPCommentGraphicsListItem(const WPComment aComment, bool aShowFullComment, BusyIndicator& aBusyIndicator, ThemeData& aThemeData)
	: WPGraphicsListItem(aBusyIndicator, aThemeData)
    , iComment(aComment)
    , iShowFullComment(aShowFullComment)
{
    
    iDate = aComment->GetShortLocalTimeString(); 
    setMinimumHeight(CommentMinimumHeight);
    //QSizePolicy policy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    //policy.setHeightForWidth(true);
    //setSizePolicy(policy);
}

WPCommentGraphicsListItem::~WPCommentGraphicsListItem()
{
}

void WPCommentGraphicsListItem::SetAvatarImage(const QPixmap &aPixmap)
{
    if (!aPixmap.isNull())
        iPixmap = aPixmap;
}

WPComment WPCommentGraphicsListItem::GetComment() const
{
    return iComment;
}

void WPCommentGraphicsListItem::Update(WPComment aComment)
{
    // XXX: no-op?
    iComment = aComment;
}

void WPCommentGraphicsListItem::ChangeViewMode()
{
    prepareGeometryChange();
    iShowFullComment = !iShowFullComment;
    SetViewMode(iShowFullComment);
}

void WPCommentGraphicsListItem::paintContent(QPainter* painter, const QStyleOptionGraphicsItem* /*option*/, QWidget* /*widget*/)
{
	int const topPadding = 10;
	
    if (!iPixmap.isNull())
    {
        // TODO default avatar from themestorage?
        painter->drawPixmap(contentXOffset(), yMargin() + topPadding, iPixmap);
    }
    
    painter->setPen(Qt::black);
    painter->setFont(*iThemeData.iListInfoFont);
    
    if (iShowFullComment)
    {
		iTitleSize.moveTop(12 + yMargin());
        iTitleSize.moveLeft(iCol2ContentXOffset);
		painter->drawText(iTitleSize,Qt::TextWordWrap, iTitle);

        painter->setFont(*iThemeData.iListContentFont);

        iCommentSize.moveTop(iTitleSize.bottom() + 5 + yMargin());
        iCommentSize.moveLeft(iCol2ContentXOffset);
        painter->drawText(iCommentSize,Qt::TextWordWrap,iCommentText);
        // TODO: Add elided texts
        painter->drawText(QPoint(iCol2ContentXOffset, iCommentSize.bottom() + 24 + yMargin()), iComment->iAuthorEmail);
        painter->drawText(QPoint(iCol2ContentXOffset, iCommentSize.bottom() + 44 + yMargin()), iComment->iAuthorUrl);
    }
    else
    {
        painter->drawText(QPoint(iCol2ContentXOffset, 24 + yMargin()), iTitle);
        painter->setFont(*iThemeData.iListContentFont);
        painter->drawText(QPoint(iCol2ContentXOffset, 56 + yMargin()), iCommentText);
        // TODO: Add elided texts
        painter->drawText(QPoint(iCol2ContentXOffset, 112 + yMargin()), iComment->iAuthorEmail);
        painter->drawText(QPoint(iCol2ContentXOffset, 132 + yMargin()), iComment->iAuthorUrl);
    }

    painter->setFont(*iThemeData.iListDetailFont);
    painter->drawText(QPoint(contentXOffset(), 112 + yMargin()), iComment->iAuthor);
    painter->drawText(QPoint(contentXOffset(), 132 + yMargin()), iDate);
}

bool WPCommentGraphicsListItem::IsBusy()
{
    return iComment->IsBusy();
}

bool WPCommentGraphicsListItem::HasError()
{
    return iComment->iError;
}

void WPCommentGraphicsListItem::handleGeomeryChange()
{
	/*
	if (!iShowFullComment)
	{
		calculateElidedTexts();
	}
	*/
	SetViewMode(iShowFullComment);
	iCol2ContentXOffset = contentXOffset() + 100;
}

void WPCommentGraphicsListItem::calculateElidedTexts()
{
    QFontMetrics titleFm(*iThemeData.iListTitleFont);
    iTitle = titleFm.elidedText(iComment->iPostTitle, Qt::ElideRight, int(width()) - 130);
    QFontMetrics commentFm(*iThemeData.iListContentFont);
    iCommentText = commentFm.elidedText(iComment->iContent, Qt::ElideRight, int(width()) - 130);
}

void WPCommentGraphicsListItem::SetViewMode(bool aShowFullComment)
{
    if (aShowFullComment)
    {
        QFontMetrics titleFm(*iThemeData.iListTitleFont);
        QFontMetrics commentFm(*iThemeData.iListContentFont);
        const int maxCommentHeight(1000);
        
        QRect titleRect(0,0,int(width()) - 130,maxCommentHeight);
        iTitleSize = titleFm.boundingRect(titleRect,Qt::TextWordWrap,iComment->iPostTitle);
        
        QRect commentRect(0,0,int(width()) - 130,maxCommentHeight);
        iCommentSize = commentFm.boundingRect(commentRect,Qt::TextWordWrap,iComment->iContent);
        
        qDebug("WP::WPCommentGraphicsListItem::SetViewMode titlesize %d x %d, commentsize %d x %d",
            iTitleSize.width(),iTitleSize.height(),iCommentSize.width(),iCommentSize.height());
        
        iTitle = iComment->iPostTitle;
        iCommentText = iComment->iContent;
        
        int height = iTitleSize.height() + iCommentSize.height() +  2 * yMargin() + 100; 
        if (height > CommentMinimumHeight)
        {
			setMinimumHeight(height);
        }
        else 
        {
			setMinimumHeight(CommentMinimumHeight);
			//setMinimumSize(geometry().width(), CommentMinimumHeight);
        }
    }
    else
    {
        calculateElidedTexts();
        setMinimumHeight(CommentMinimumHeight);
        //setMinimumSize(geometry().width(), CommentMinimumHeight);
    }
}

