#ifndef WPGRAPHICSLIST_H
#define WPGRAPHICSLIST_H

#include <QGraphicsWidget>
#include <QGraphicsView>
#include <QList>

#include "busyindicator.h"
#include "themedatastorage.h"
#include "WPData.h"

class QLabel;
class QVBoxLayout;
class QPropertyAnimation;

class PopupMenu;
class CommentsGraphicsView;
class WPGraphicsListItem;
class WPCommentGraphicsListItem;
class WPNetworkEngine;
class QGraphicsProxyWidget;
class QGraphicsLinearLayout;
class PannableView;
class BlogView;
class WPKeyDispatcher;

class WPGraphicsList : public QGraphicsWidget
{
	Q_OBJECT
	
public slots:
	void setFocus();
	
protected:
	WPGraphicsList(BlogView* aBlogView, const QGraphicsView& aGraphicsView,
	        PannableView* aPannableView);
	virtual ~WPGraphicsList() {}
	
protected slots:
	void ShowContextMenu(WPGraphicsListItem* aItem, QPoint /*aPos*/);

    void MenuHiding();
    void ContextMenuHidden();

private:
    void resizeEvent(QGraphicsSceneResizeEvent* event);
    QPoint getMenuLocation(QPoint itemPoint);
    void keyPressEvent( QKeyEvent * event  );
    void keyReleaseEvent( QKeyEvent * event );
	
private: // Pure virtual functions
    virtual void PopulatePopupMenuForCurrentItem(PopupMenu* aPopupMenu) = 0;
    virtual int FirstFocusableItemIndex() = 0;
	
protected:
	QList<WPGraphicsListItem*> iItems;
	WPGraphicsListItem* iCurrentItem;
	QGraphicsLinearLayout* iLayout;

	BusyIndicator& iBusyIndicator;
	ThemeData& iThemeData;
	
	static const int KItemSpacing;
	static const int KTitleDefaultHeight;
	
private:
	const QGraphicsView&    iGraphicsView;
	WPKeyDispatcher*        iKeyDispatcher;
	
	PopupMenu* iPopupMenu;
	
#ifndef Q_WS_MAEMO_5
	QPropertyAnimation* iAnimation;
	bool  iMenuHiding;

#endif /* Q_WS_MAEMO_5 */
	
	PannableView* iPannableView;
    
private:
    
};

#endif // WPGRAPHICSLIST_H
